st7/
;**************************************************************************************************
; LITEcomp - aplikacje
; Termometr dwupunktowy z ukadami DS18B20
;**************************************************************************************************
    .NOLIST	
		#INCLUDE "st7flite19.inc"
		#INCLUDE "1wire.inc"
		#INCLUDE "LCD.inc"
		#include "writedecimal.inc"
		#include "iButton.inc"
    .LIST  

;**************************************************************************************************
; obszar pamici danych RAM
;**************************************************************************************************		
		BYTES
		segment	'ram0'
tmplsb		DS.B	1
tmpmsb		DS.B	1


TempSign	DS.B 	1
TempInt		DS.B 	1
TempFract	DS.B 	1

;**************************************************************************************************
; obszar pamieci danych EEPROM	
;**************************************************************************************************
		WORDS 					
		SEGMENT	'eeprom'
inAddress		DS.B	8
outAddress 	DS.B	8

;**************************************************************************************************
; obszar pamieci programu		
;**************************************************************************************************
		WORDS	
		segment 'rom'
		
fractConst	DC.B '0','1','1','2','3','3','4','4','5','6','6','7','8','8','9','9'
stopien			DC.B	6, 9, 9, 6, 32, 32, 32, 32, 0

tekstIn			STRING 	"IN :",0
tekstOut		STRING	"OUT:",0
tekstRegIn	STRING	"IN Sensor",0
tekstRegOut	STRING	"OUT Sensor",0

;**************************************************************************************************
; poczatek programu gwnego
;**************************************************************************************************
._reset
		CALL	init										; inicjalizacja procesora
		CALL	lcdinit									; inicjalizacja wywietlacza

		LD		A, #64									;
		CALL	writecommand						;
		WTXT  stopien									;	zapis do pamici CGRAM symbolu "stopie"

		BTJT	PADR, #0, next1					; jeli przycisk S1 nie jest wcinity to skok do next1
		CALL	regInAddress						; w przeciwnym razie wywoanie procerury 
		BTJF	PADR, #0, *							; oczekiwanie na zwolnienie przycisku
next1
		BTJT	PADR, #2, next2					; jeli przycisk S2 nie jest wcinity to skok do next2
		CALL	regOutAddress						; w przeciwnym raze wywoanie procedury
		BTJF	PADR, #2, *							; oczekiwanie na zwolnienie przycisku
next2
		LD		A,#1										;
		CALL	writecommand						; CLEAR SCREEN
		
		WTXT	tekstIn									; wywietlenie tekstu "IN :"
		LD		A, #SECLINEADDR					; druga linia
		CALL	lcdsetaddress						;
		WTXT	tekstOut								; wywietlenie tekstu "OUT :"
	
;**************************************************************************************************
; gwna ptla programu	
;**************************************************************************************************
mainloop
		LD		A, #inAddress.h					; 
		LD		{ptrRead+0}, A					; Zaadowanie do wskanika odczytu
		LD		A, #inAddress.l					; adresu czujnika wewntrznego
		LD		{ptrRead+1}, A					; 

		CALL	gettemp 								; odczyt temperatury wewntrznej
		CALL	convertTemp							; konwersja wyniku pomiaru		
		LD		A, #5										; pita kolumna
		CALL	lcdsetaddress						; ustawienie wsprzdnych wywietlacza
		CALL	displaytemp							; wywietlenie temperatury
		
		LD		A, #outAddress.h				;
		LD		{ptrRead+0}, A					; Zaadowanie do wskanika odczytu
		LD		A, #outAddress.l				; adresu czujnika zewntrznego
		LD		{ptrRead+1}, A					;

		CALL	gettemp 								; odczyt temperatury zewntrznej
		CALL	convertTemp							; konwersja wyniku pomiaru
		LD		A, #5										; pita kolumna
		ADD		A, #SECLINEADDR					; drugi wiersz
		CALL	lcdsetaddress						; ustawienie wsprzdnych wywietlacza
		CALL	displaytemp							; wywietlenie temperatury
		
		JRA		mainloop 								; skok na pocztek ptli
		
;**************************************************************************************************
; Procedura inicjalizacji
;**************************************************************************************************
init
		LD		A, #numer.h							; zainicjowanie wskanika numprt
		LD		{numptr+0}, A						;
		LD		A, #numer.l							;
		LD		{numptr+1}, A						;
		
		LD		A, #$0C									; zainicjowanie timera
		LD		ATRH, A									; odlicza 1000 cykli
		LD		A, #$18									; czyli 1sekund 
		LD		ATRL, A									;	przy taktowaniu 1KHz
		
		BSET	ATCSR, #4								; szybkie przeadowanie licznika
		BTJF	ATCSR, #2, *						; z pen czstotliwoci taktowania
		BRES	ATCSR, #4								;
		RET
		
;**************************************************************************************************
; Proceruda rejestracji numeru ukadu mierzcego temperature wewntrzn
;**************************************************************************************************
regInAddress
		CLR	A
		CALL	lcdsetaddress
		WTXT	tekstRegIn
		LD		A, #inAddress.h
		LD		{ptrWrite+0}, A
		LD		A, #inAddress.l
		LD		{ptrWrite+1}, A
		CALL	registerCode
		RET
		
;**************************************************************************************************
; Procedura rejestracji numeru ukadu mierzcego temperature zewntrzn
;**************************************************************************************************
regOutAddress
		CLR	A
		CALL	lcdsetaddress
		WTXT	tekstRegOut
		LD		A, #outAddress.h
		LD		{ptrWrite+0}, A
		LD		A, #outAddress.l
		LD		{ptrWrite+1}, A
		CALL	registerCode
		RET
		
;**************************************************************************************************
; Procedura oczekiwania na zakoczenie pomaru (1 sekunda)
;**************************************************************************************************
waitTimer
		LD		A, #%00001000 						; wczenie timera F=1KHz
		LD		ATCSR, A
		BTJF	ATCSR, #2, *							; oczekiwanie na przepenienie
		CLR		ATCSR											; zatrzymanie timera
		RET
		
;**************************************************************************************************
; Procedura pomiaru temperatury
;**************************************************************************************************
.gettemp
		CALL 	owreset										; reset magistrali 1wire
		JRC		gettemp										;	jeli brak impulsu obecnoci to skok do gettemp
		LD 		A, #$55	
		CALL 	owwrite										; zapis komendy MATCH ROM
		CALL	writenumer								; zapis adresu ukadu DS18B20
		LD		A, #$44
		CALL	owwrite										; zapis komendy CONVERT
		CALL	waitTimer									; oczekiwanie na zakoczenie pomiaru
		CALL 	owreset 									; reset magistrali 1wire
		LD 		A, #$55		
		CALL 	owwrite										; zapis komendy MATCH ROM
		CALL	writenumer								; zapis adresu ukadu DS18B20
		LD 		A, #$BE
		CALL 	owwrite 									; zapis komendy READ SCRATCHPAD
		CALL 	owread
		LD 		tmplsb, A 								; odczyt modszego bajtu wyniku pomiaru
		CALL 	owread
		LD 		tmpmsb, A									; odczyt starszego bajtu wyniku pomiaru
		RET
		
;**************************************************************************************************
; Konwersja wyniku pomiaru 
;**************************************************************************************************
.convertTemp
		LD		A, #'+'										; zakadamy, e temperatura jest dodatnia
		LD		TempSign, A								; zapamitanie znaku temperatury
ct1		
		LD		A, tmpmsb									;
		JRMI	cTn												; jeli ujemna to skok do cTn
		SWAP	A													; w przeciwnym razie mnoenie przez 16
		AND		A, #$F0										; zamaskowanie nieznaczcych bitw
		LD		tmpmsb, A									; i zapamitanie w tmpmsb
		LD		A, tmplsb									; zaadowanie mniej znaczcego bajtu wyniku pomiaru do A
		SWAP	A													; dzielenie przez 16			
		AND		A, #$0F										;	zamaskowanie nieznaczcych bitw
		ADD		A, tmpmsb									; zsumowanie uprzednio obliczonych wartoci
		LD		TempInt, A								; zapamitanie czci cakowitej wyniku pomiaru
		LD		A, tmplsb									; zaadowanie mniej znaczcego bajtu wyniku pomiaru do A
		AND		A, #$0F										; zamaskowanie nieznaczcych bitw
		LD		TempFract, A							; zapamitanie czci uamkowej wyniku pomiaru
		RET															; powrt z procedury
cTn
		LD		A, #'-'										;	temperatura ujemna
		LD		TempSign,A								; zapamitanie znaku wyniku pomiaru
		CPL		tmpmsb										; * konwersja z kodu U2 na NKB *
		CPL		tmplsb										; negacja
		LD		A, tmplsb									;
		ADD		A, #1											; LSB + 1
		LD		tmplsb, A									;
		CLR		A													;
		ADC		A, tmpmsb									; MSB + C (dodawanie 16-bitowe)
		LD		tmpmsb, A									;  
		JRA		ct1												; dalej mozna potraktowac jak liczbe dodatni


;**************************************************************************************************
; Procedura wyswietlajca temperature na wyswieltaczu
;**************************************************************************************************
.displaytemp
		LD		A, TempSign								; wywietlenie znaku
		CALL	writedata
		LD		A, TempInt							
		CALL	writedecimal							; wywietlenie czsci cakowitej	
		LD		A, #'.'
		CALL	writedata									; wywietlenie kropki
		LD		X, TempFract
		LD		A, (fractConst,X)
		CALL	writedata									; wywietlenie czci uamkowej
		LD		A, #8
		CALL	writedata									; wywietlenie symbolu "stopie"
		LD		A, #'C'
		CALL	writedata									; wywietlenie litery "C"
		LD		A, #' '
		CALL	writedata									; wywietlenie spacji
		RET

    END
;**************************************************************************************************
; Koniec pliku main.asm
;**************************************************************************************************		

