//--------------------------------------------------------------------------------------------------
// LITEcomp - aplikacje
// Woltomierz jednokanaowy z pamieci wynikw pomiarw
//--------------------------------------------------------------------------------------------------

#ifdef __RAISONANCE__
	#include <st7lite1.h>
	#include <intrins.h>
#endif

#ifdef __CSMC__
	#include <io7flit1.h>
	#define code const
#endif


#include "lcd.h"

#define EE_BASE	0x1000
#define EE_SIZE	16

#define EOC		7
#define ADON	5
#define SLOW	3

#define SW1		(1<<0)
#define SW2		(1<<2)

#define BUZZ	(1 << 0)

volatile int volts = 0;
volatile char eeIndex = 0, readIndex = 0;
code char adresstronybtc[] = "www.btc.pl";
//--------------------------------------------------------------------------------------------------
//
// Funkcja wywietlajca numer pozycji w pamici
//
//--------------------------------------------------------------------------------------------------
void DisplayPosition(unsigned char val)
{
LCD_WriteData(((val%100)/10)+'0');
LCD_WriteData(((val%100)%10)+'0');
}
//--------------------------------------------------------------------------------------------------
//
// Funkcja waczajca buzzer. Argument funkcji jest liczb "pikni"
//
//--------------------------------------------------------------------------------------------------
void Beep(unsigned char beep)
{
volatile int i;
for( ; beep > 0; beep--)// ilo "pikni"
  {
  PBDR &= ~BUZZ; // wczenie buzzera
  for(i = 0; i < 10000; i++);  
  PBDR |= BUZZ;  // wyczenie buzzera
  for(i = 0; i < 10000; i++); 
  }
}
//--------------------------------------------------------------------------------------------------
//
// Funkcja odczytujca warto napiecia na wybranym wejciu analogowym
//
//--------------------------------------------------------------------------------------------------
unsigned int GetADC(unsigned char channel)
{
unsigned int value;
ADCDRL |= (1 << SLOW);// fADC = 4MHz
ADCCSR = channel; // wybr kanau
ADCCSR |= (1 << ADON); // wczenie przetwornika
while((ADCCSR&(1 <<  EOC))==0);//oczekiwanie na zakoczenie konwersji
value = (ADCDRL & 0x02);//
value |= (ADCDRH << 2); // odczytanie wyniku
ADCCSR &= ~(1 << ADON); // wyaczenie przetwornika
return value;
}
//--------------------------------------------------------------------------------------------------
//
// Funkcja wywietlajca warto napicia w woltach.
//
//--------------------------------------------------------------------------------------------------
void DisplayVolts(unsigned int voltage)
{
float napiecie;
unsigned char jednostki,dziesietneCzesci,setneCzesci;
unsigned int reszta;

napiecie = voltage * 4.8828125; // konwersja na miliwolty

// rozbicie na poszczeglne cyfry
jednostki = (unsigned int)(napiecie / 1000);
reszta = (unsigned int)napiecie % 1000;
dziesietneCzesci = (unsigned char)(reszta / 100);
setneCzesci = (unsigned char)((reszta % 100)/10);

// wywietlenie wyniku
LCD_WriteData(jednostki + '0');
LCD_WriteData('.');
LCD_WriteData(dziesietneCzesci + '0');
LCD_WriteData(setneCzesci + '0');
LCD_WriteData('V');
}
//--------------------------------------------------------------------------------------------------
//
// Funkcja zapisujca do pamieci EEPROM wynik pomiaru napicia
//
//--------------------------------------------------------------------------------------------------
void WriteVoltsToEEPROM(void)
{
if(eeIndex < EE_SIZE) // 
  {
  EECSR = 2; // ustawiamy bit E2LAT -> zapis
  *((unsigned int *)(EE_BASE + (eeIndex*2))) = volts;  // zapis do buforw
  EECSR |= 1; // ustawiamy bit E2PGM -> wyzwolenie operacji zapisu
  while((EECSR & 1)!=0); // oczekiwanie za zakoczenie procesu zapisu
  Beep(1); // sygnalizacja zapisu
  eeIndex++; // nastpna pozycja
  readIndex = eeIndex; // przygotowanie do odczytu
  }
else
  {
  eeIndex = 0;// ustawienie pzycji na doln granic obszaru pamici EEPROM
  Beep(2); // sygnalizacja osignicia granicy obszaru
  }
}
//--------------------------------------------------------------------------------------------------
//
// Funkcja odczytujca z pamici EEPROM zapamitany wynik pomiaru
//
//--------------------------------------------------------------------------------------------------
void ReadVoltsFromEEPROM(void)
{
unsigned char i;

readIndex--; // poprzednio zapamitany pomiar

if(readIndex < 0) // jeli przekroczono doln granic obszaru pamici EEPROM
   {
	readIndex = EE_SIZE; // ustawienie pozycji na grn granic obszaru
	Beep(2); // sygnalizacja osignicia granicy obszaru
   }
else // w przeciwnym razie
  {
  	EECSR = 0; // zerowanie bitu E2LAT -> odczyt
  	volts = *((unsigned int *)(EE_BASE + ((readIndex)*2))); // odczyt z pamici
	LCD_GoTo(0,0); // ustawienie wsprzdnych
	DisplayVolts(volts); // wywietlenie odczytnej wartoci napicia
	LCD_GoTo(12,0); // ustawienie wsprzdnych
	LCD_WriteText("M>");
	DisplayPosition(readIndex); // wywietlenie numeru pozycji w pamici
	Beep(1); // sygnalizacja odczytu
  }
}
//--------------------------------------------------------------------------------------------------
//
// Funkcja sprawdzajaca stan przyciskw 
//
//--------------------------------------------------------------------------------------------------
void CheckKeys(void)
{
if((PADR & SW1)==0) // jeli wcinito przycisk S1
  {
  WriteVoltsToEEPROM(); // zapis napicia do pamieci EEPROM
  while((PADR & SW1)==0); // oczekiwanie na zwolnienie przycisku
  }
if((PADR & SW2)==0) // jeli wcinito przycisk S2
  {
  ReadVoltsFromEEPROM(); // odczyt napicia z pamici EEPROM
  while((PADR & SW2)==0); // oczekiwanie na zwolnienie przycisku
  }                       
}
//--------------------------------------------------------------------------------------------------
//
// Gwna funkcja programu 
//
//--------------------------------------------------------------------------------------------------
int main(void)
{
PAOR = 0xFF;
PBDR = BUZZ;
PBDDR = BUZZ;
LCD_Init(); // inicjalizacja LCD
LCD_GoTo(3,1); // ustawienie wsprzdnych
LCD_WriteText(adresstronybtc); // wywietlenie tekstu
 
while(1) // ptla nieskoczona
 {
  volts = GetADC(1); // pomiar napicia
  LCD_GoTo(0,0); // ustawienie wsprzdnych
  DisplayVolts(volts); // wywietlenie napiecia
  CheckKeys(); // sprawdzenie przyciskw
  LCD_GoTo(12,0); // ustawienie wsprzdnych
  LCD_WriteText("M:"); // wywietlenie tekstu
  DisplayPosition(eeIndex); // wywietlenie pozycji pamici
 }
return 0;
}
//--------------------------------------------------------------------------------------------------
//
// Koniec pliku main.c
//
//--------------------------------------------------------------------------------------------------
